//*******************************************************************
// Enable.h
// Routines to enable and disable interrupts
//
// Created Sept 14, 2006
// Jim Patchell
//*******************************************************************

#ifndef ENABLE__H
#define ENABLE__H

#include "intrinsics.h"

#pragma inline
extern long Disable(void)
{
	//--------------------------------------------------------------------------
	// Disable()
	//
	// This function disables interrupts and returns the previous status of
	// the interrupts before the disable operation
	//
	// Return Value:
	//	If the interrupt bit is set, this indicates that the interrupt
	// was enable, if bit is cleared, interrupt was disabled
	//--------------------------------------------------------------------------
	register long retval;

	retval = __get_CPSR();
	__set_CPSR(retval | 0x0c0);
	return (retval ^ 0xc0) & 0x0c0;
}

#pragma inline
extern void Enable(long sr)
{
	//---------------------------------------------------------------------------
	// Enable(sr)
	//
	// This function is used to enable interrupt bits.
	//
	// paramter:
	//	sr.......mask to enable interrupts.
	//				if bit is set, interrupt will be enabled
	//				if bit is zero, not change to interrupt status will occure
	//
	// it should be noted that in the ARM interrupts are enabled when the 
	// coresponding bit is cleared, and disabled when set...so, don't get
	// too confused by the fact that these functions do it backwards from that
	//---------------------------------------------------------------------------
	register long t;

	t = __get_CPSR();
	t &= ~sr;
	__set_CPSR(t);
}
#endif
