/****************************************************************************************
** RS232 IO routines
** Created Sept 14, 2006
** Jim Patchell
**
**--------------------------------------------------------------------------------------
** Revision History....
**
** Need to enable UART Receive Timeout Interrupt 10-6-2006
**
****************************************************************************************/
#include <stdio.h>
#include "rs232.h"
#include "91x_uart.h"
#include "91x_vic.h"
#include "Enable.h"
#include "91x_scu.h"
#include "cio.h"
#include "BlockAlloc.h"

long Uart0IrqCount,Uart1IrqCount,Uart2IrqCount;

IOREC RecRecs[3],XMitRecs[3];	//IO record Lookups
static char RBuff0[RS232_BUFFSIZE];
static char RBuff1[RS232_BUFFSIZE];
static char RBuff2[RS232_BUFFSIZE];

static char TBuff0[RS232_BUFFSIZE];
static char TBuff1[RS232_BUFFSIZE];
static char TBuff2[RS232_BUFFSIZE];

static IOCB *StdIO;

static char *TBLut[3] = {
	TBuff0,
	TBuff1,
	TBuff2
};

static char *RBLut[3] = {
	RBuff0,
	RBuff1,
	RBuff2
};

static UART_InitTypeDef Temp_Init;

//------------------------------------------------------------------------------
// Standard I/O port...allows us to use printf...
//******************************************************************************

void SetPutChar(IOCB *I)
{
	//**************************************************************************
	// This function is used to set the I/O chananel to use for standard output
	// parameter:
	//	I........pointer to the IO control block to use as standard output
	//**************************************************************************
	StdIO = I;	//set StdIO to desired channel
}

int putchar(int c)
{
	//**************************************************************************
	// redefine putchar...this will override the function that is in the
	// starndard c library
	// parameters:
	//	c..............character to output
	//	
	// return value:
	//	returns the character that was sent
	//**************************************************************************

	if(StdIO)
	{
		if(c == '\n')
			CioPutC(StdIO,'\r');
		CioPutC(StdIO,c);
	}
	return (int)(unsigned char) c;
}


static UART_TypeDef *HandleToUart(short Handle)
{
	//--------------------------------------------------------------------------
	// This function is used to convert the UART handle into a pointer to
	// a UART_Typedef structure.
	//
	// parameter:
	//	Handle.....Handle of uart...
	//				0 -> Uart 0
	//				1 -> Uart 1
	//				2 -> Uart 2
	//--------------------------------------------------------------------------
	UART_TypeDef *retval;

	switch(Handle)
	{
		case RS232PORT0:
			retval = UART0;
			break;
		case RS232PORT1:
			retval = UART1;
			break;
		case RS232PORT2:
			retval = UART2;
			break;
		default:
			retval = NULL;
			break;
	}
	return retval;
}

static void EnableUartPins(short Handle)
{
	//**************************************************************************
	// This function configure the pins on the chip
	// for the Uart
	//
	// parameter:
	//	Handle.....port number to configure
	//**************************************************************************
	switch(Handle)
	{
		case RS232PORT0:
			SCU->GPIOIN[5] |= BIT(1);		//P5.1
			SCU->GPIOOUT[3] |= (0x03 << 8);	//P3.4
			GPIO5->DDR |= BIT(1);
			break;
		case RS232PORT1:
			SCU->GPIOIN[3] |= BIT(2);		//P3.2 RxD
			SCU->GPIOOUT[3] |= (0x2 << 6);	//P3.3 TxD
			GPIO3->DDR |= BIT(3);
			break;
		case RS232PORT2:
			SCU->GPIOIN[3] |= BIT(1);		//P3.1 RxD
			SCU->GPIOOUT[3] |= (0x03 << 10);		//P3.6 TxD
			GPIO3->DDR |= BIT(5);
			break;	
	}
}

static const u32 APB_Uart[3] = {
	__UART0,
	__UART1,
	__UART2
};

static const u16 VIC_LineLUT[3] = {
	UART0_ITLine,
	UART1_ITLine,
	UART2_ITLine
};



static void RInit(short Handle,long Baudrate,short parity,short bits, short Priority)
{
	//--------------------------------------------------------------------------
	// Initialize a UART
	//
	// Parameter:
	//	Handle.......Handle of Uart to initialize
	//	Baudrate.....Desired Uart Bitrate
	//	parity.......Type of parity check to use
	//	bits.........Number of bit in word
	//
	// We default to 1 stop bit
	//--------------------------------------------------------------------------
	int i;
	volatile char c;
	ECB *RSem;
	ECB *TSem;
	IOREC *pRR,*pTR;
	pRR = &RecRecs[Handle];
	pTR = &XMitRecs[Handle];
	char *p = NewBlock();
	SCU_APBPeriphClockConfig(APB_Uart[Handle],ENABLE);
	SCU_UARTIrDASelect(pRR->Uart,SCU_UARTMode_UART);
	EnableUartPins(Handle);
	pRR->Uart = HandleToUart(Handle);	//initialize Recieve descriptor
	pRR->Head = 0;
	pRR->Tail = 0;
	pRR->Size = RS232_BUFFSIZE;
	pRR->nChar = 0;
	pRR->Errors = 0;
	pRR->pBuff = RBLut[Handle];
	sprintf(p,"COMRSEM%d",Handle);
	RSem = NewSemaphore(0,0,p);
	pRR->Sem = RSem;
	pRR->Block = 0;
	pTR->Uart = RecRecs[Handle].Uart;	//initialize Transmit descriptor
	pTR->Head = 0;
	pTR->Tail = 0;
	pTR->Size = RS232_BUFFSIZE;
	pTR->nChar = 0;
	pTR->pBuff = TBLut[Handle];
	pTR->Errors = 0;
	sprintf(p,"COMTSEM%d",Handle);
	TSem = NewSemaphore(0,0,p);
	pTR->Sem = TSem;
	sprintf(p,"COMTBLOCK%d",Handle);
	pTR->Block = NewSemaphore(1,0,p);
	//initialize hardware
	UART_DeInit(pRR->Uart);	//reset uart
	UART_Cmd(pRR->Uart,ENABLE);
	UART_StructInit(&Temp_Init);
	Temp_Init.UART_WordLength = bits;
	//Temp_Init.UART_Parity = parity;
	Temp_Init.UART_BaudRate = Baudrate;
	UART_Init(pRR->Uart,&Temp_Init);	//initialize uart
	while(((pRR->Uart->FR & 0x10) == 0))	//while rx fifo not empty
		c = pRR->Uart->DR;

	for(i=0;i<11;++i)
		UART_ClearITPendingBit(pRR->Uart,BIT(i));
	VIC_Config(VIC_LineLUT[Handle], VIC_IRQ,Priority);		//configure Uart
	VIC_ITCmd(VIC_LineLUT[Handle],ENABLE); 
	UART_ITConfig(pRR->Uart,UART_IT_Receive,ENABLE);	//enable receive interrupts
	UART_ITConfig(pRR->Uart,UART_IT_Transmit,ENABLE);
	UART_ITConfig(pRR->Uart,UART_IT_ReceiveTimeOut,ENABLE);
	DeleteBlock(p);
}

static void AddBuffer(IOREC *pI,short c)
{
	//------------------------------------------------
	// add a character to a buffer
	//
	// parameter:
	//	pI.........pointer to buffer to add character to
	//	c..........character to add
	//
	// if there is no room...character is dropped and
	// error count is incremented
	//-------------------------------------------------
	if(pI->nChar < pI->Size)	//is there space?
	{
		pI->pBuff[pI->Head++] = (char) c;
		if(pI->Head == pI->Size) pI->Head = 0;
		pI->nChar++;
	}
	else
		pI->Errors++;
}

void RRecIrqHandler(short Handle)
{
	//-----------------------------------------------------------------------------
	// Recieve Interrupt handler
	//
	// parameter:
	//	Handle.....handle of rs232 port to handle incoming char interrupt for
	//-----------------------------------------------------------------------------
	char c;
	IOREC *pI = &RecRecs[Handle];

	while(((pI->Uart->FR & 0x10) == 0))	//while rx fifo not empty
	{
		if(pI->nChar != pI->Size)
		{
			c = UART_ReceiveData(pI->Uart);
			pI->pBuff[pI->Head++] = c;	//put character into buffer
			if(pI->Head == pI->Size) pI->Head = 0;	//wrap head pointer
			pI->nChar++;
			PostSemaphore(pI->Sem,0);
		}
		else
			c = UART_ReceiveData(pI->Uart);
	}
}

void RSendIrqHandler(short Handle)
{
	//--------------------------------------------------------------------------------
	// Transmit Interrupt Handler
	//
	// parameter:
	//	Handle.....handle of rs232 port to handle outgoing char interrupt for
	//--------------------------------------------------------------------------------
	IOREC *pI = &XMitRecs[Handle];
	short loop = 1;

	while( ((pI->Uart->FR & 0x20) == 0) && loop)	//is output fifo not full?
	{
		if(pI->nChar == 0)
		{
			pI->Uart->ICR |= 0x20;	//clear xmit irq
			loop = 0;	//terminate loop
		}
		else
		{
			pI->Uart->DR = pI->pBuff[pI->Tail++];
			if(pI->Tail == pI->Size) pI->Tail = 0;	//wrap tail pointer
			pI->nChar--;
		}
	}	//end of while statement
}

//------------------------------------------------------------------------------
// 
// Device Driver Interface
//
//------------------------------------------------------------------------------

static const char DevName[] = "COM";

static long Open(IOCB *I)
{
	//--------------------------------------------------------------------------
	// Open
	// Open up an RS232 com port
	// parameter:
	//	I.........pointer to the IO control block for the device
	//
	// return value:
	//	0 for success, other device dependent value otherwise
	//--------------------------------------------------------------------------
	long retval=0;

	return retval;
}

static long Close(IOCB *I)
{
	//--------------------------------------------------------------------------
	// Close
	// Close up an RS232 com port
	// parameter:
	//	I.........pointer to the IO control block for the device we want to close
	// return value:
	//	returns 0 on success
	//--------------------------------------------------------------------------
	long retval=0;

	return retval;
}

static long GetC(IOCB *I)
{
	//--------------------------------------------------------------------------
	// GetC
	// Get a character from an RS232 channel
	//
	// parameter:
	//	Handle.....Handle of RS232 port to use
	//
	// Return value:
	//	If there are not chars availiable, return -1
	//	else return a character from the buffer
	//--------------------------------------------------------------------------
	long retval;

	IOREC *pI = &RecRecs[I->devnum];
	int sr;

	PendSemaphore(pI->Sem,0);	//check for data in buffers, block otherwise
	sr = Disable();
	if(pI->nChar)
	{
		retval = (int)pI->pBuff[pI->Tail++];
		if(pI->Tail == pI->Size) pI->Tail = 0;
		pI->nChar --;
	}
	else
		retval = -1;		//error, no chars
	Enable(sr);
	return retval;
}

static long Read(IOCB *I,char *b,int n)
{
	//--------------------------------------------------------------------------
	// Read
	//	Read a buffer of characters from a RS232 com port
	// parameter:
	//	I.........pointer to the IOCB to for the COM port to read
	//	b.........pointer to buffer to put characters into to
	//	n.........number of characters to read
	// return value
	//	return the number of characters read
	//--------------------------------------------------------------------------
	int i;
	for(i=0;i<n;++i) b[i] = (char)GetC(I);
	return i;
}

static long PutC(IOCB *I,int c)
{
	int retval = 0;
	//--------------------------------------------------------------------------
	// PutC
	// Put a character to an RS232 port
	// If there is no space...the character is lost
	//
	// parameter:
	//	I...........Pointer to IOCB of COM port to write character to
	//	c...........Character to send
	// return value:
	//	returns 0 on success
	//---------------------------------------------------------------------------

	IOREC *pI = &XMitRecs[I->devnum];
	int sr;

	sr = Disable();
	if(pI->nChar)	//are there characters in the buffer?
	{
		AddBuffer(pI,c);	//well, add more
	}
	else if ((pI->Uart->FR & 0x20) == 0)	//is there room in fifo?
		pI->Uart->DR = c;					//add directly to fifo
	else
		AddBuffer(pI,c);	//well, add to buffer then...
	Enable(sr);
	return retval;
}

static long Write(IOCB *I,char *b,int n)
{
	//--------------------------------------------------------------------------
	// Write
	// Write a buffer of characters to a COM port.  This function is blocked
	// so that only one task can use it at a time
	// parameter:
	//	I..........pointer to IOCB of COM port to write
	//	b..........pointer of characters to write
	//	n..........number of characters to write
	// return value:
	//	returns 0 on success
	//--------------------------------------------------------------------------

	long retval = 0;
	int i;
	IOREC *pI = &XMitRecs[I->devnum];

	PendSemaphore(pI->Block,0);
	for(i = 0;i<n;++i) PutC(I,b[i]);
	PostSemaphore(pI->Block,0);
	return retval;
}

static long Status(IOCB *I,int m)
{
	long retval;
	//--------------------------------------------------------------------------
	// Return Status of RS232 port
	//
	// parameter:
	//	I..........Handle of RS232 port to get status of
	//	m..........Type of status to get:
	//				RS232_TYPE_RECIEVE->Number of chars in recieve buffer
	//				RS232_TYPE_TRANSMIT->Free Space in transmit buffer
	//--------------------------------------------------------------------------
	int sr;

	sr = Disable();
	switch(m)
	{
		case RS232_TYPE_RECIEVE:
			retval = RecRecs[I->devnum].nChar;		//return the number of chars in recieve buffer
			break;
		case RS232_TYPE_TRANSMIT:
			//return free space to add characters in transmit buffer
			retval = XMitRecs[I->devnum].Size - XMitRecs[I->devnum].nChar;
			break;
	}
	Enable(sr);
	return retval;
}

static long XIO(IOCB *I,int cmd,va_list argp)
{
	//--------------------------------------------------------------------------
	// XIO
	// Extended IO functions.  These are functions that do not fall into any of
	// the other IO vectors above and are device dependent
	//
	// parameters:
	//	I..........pointer to IO Control block for device
	//	cmd........Command Function to perform
	//	argp.......pointer to argument list of parameters...device dependent
	// return value:
	//	device dependent long value
	//--------------------------------------------------------------------------

	long retval=0;

	int p1,p2,p3;
	char *pC;

	switch(cmd)
	{
		case 0:	//get two int params
			p1 = va_arg(argp,int);
			p2 = va_arg(argp,int);
			printf("XIO of %s:cmd=%d P1=%d P2=%d\n",DevName,cmd,p1,p2);
			break;
		case 1:	//get three int params
			p1 = va_arg(argp,int);
			p2 = va_arg(argp,int);
			p3 = va_arg(argp,int);
			printf("XIO of %s:cmd=%d P1=%d P2=%d P3=%d\n",DevName,cmd,p1,p2,p3);
			break;
		case 2:	//get char * and two ints
			pC = va_arg(argp,char *);
			p2 = va_arg(argp,int);
			p3 = va_arg(argp,int);
			printf("XIO of %s:cmd=%d pC=%08x P2=%d P3=%d\n",DevName,cmd,pC,p2,p3);
			break;
	}
	return retval;
}

//---------------------------------------------
// Vector Table
//---------------------------------------------

static H_JVEC Dev1IOVectors = {
	Open,
	Close,
	GetC,
	Read,
	PutC,
	Write,
	Status,
	XIO,
	RS232Init
};

long RS232Init(void)
{	
	RInit(0,57600,0,UART_WordLength_8D,2);
	RInit(1,57600,0,UART_WordLength_8D,4);
	RInit(2,57600,0,UART_WordLength_8D,3);
	CioAddHandler((char *)DevName,&Dev1IOVectors);
	return 0;
}
