/*****************************************************************************
** Test tasks for RTOS
*****************************************************************************/
#include "stdio.h"
#include "stdlib.h"
#include "task.h"
#include "cio.h"
#include "enable.h"
#include "main.h"
#include "91x_lcd.h"
#include "rs232.h"
#include "BlockAlloc.h"

int TCBIndex;
int MaxTCB;
ECB TS_Led,ADC_done;
IOCB *Com0 = 0;

TCB TtTcb,AdcTcb,Rs232Tcb,Rs232Tcb1,Rs232Tcb2,IdleTcb;
long TtStack[128];
long AdcStack[128];
long IdleStack[128];
long RS232Stack[128];
long RS232Stack1[128];
long RS232Stack2[128];

//-------------------------------------------------
// task test code
//-------------------------------------------------

/*******************************************************************************
* Function Name  : LedShow_Init
* Description    : Configure the leds pins as output pushpull
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LedShow_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
 
	/* GPIO3 pins configuration */
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_Alternate=GPIO_OutputAlt1;
	GPIO_Init(GPIO3, &GPIO_InitStructure);
  
  /* GPIO6 pins configuration */
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
	GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
	GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull;
	GPIO_InitStructure.GPIO_Alternate=GPIO_OutputAlt1;
	GPIO_Init(GPIO6, &GPIO_InitStructure);
} 

unsigned short LedShow(unsigned short k)
{
	static unsigned short Dir = 0;

    GPIO_Write(GPIO3,0x0ff);
    GPIO_Write(GPIO6,0x0ff);
    GPIO_WriteBit(GPIO3,k & 0xff,Bit_RESET);
    GPIO_WriteBit(GPIO6,k>>8,Bit_RESET);
	if(Dir == 0)
       k <<= 1;
	else
		k >>= 1;
	if(k == 0x8000)
		Dir = 1;
	else if (k == 1)
		Dir = 0;
	return k;
}

void TestTask(void *a)
{
	unsigned short k;

	k = (unsigned short)a;
	/* Initialize LedShow */
	LedShow_Init();
	while(1)
	{
		k = LedShow(k);
		PendSemaphore(&TS_Led,20);
	}
}

void ADCTask(void *a)
{
	extern volatile int ADCTick;
	static unsigned char buffer[128];
	int Conversion_Value;

	if(Com0 == NULL) Com0 = CioOpen("COM0",CIO_READ_ONLY | CIO_WRITE_ONLY);
	SetPutChar(Com0);
	a = a;	//keep compiler from barking
  /* Initialize LCD driver */
	LCD_Init();
	LCD_Clear();
  /* Display " Welcome to the " message on LCD: Line 1 */ 
	LCD_SendStringByStep(" Welcome to the ", 1, 1);
	TimeDelay(1000);   
	/* Display "   STR912 Demo   " message on LCD: Line 2 */ 
	LCD_SendStringByStep("  STR912 Demo   ", 2, 1); 
	/* Wait the end of Osiris wave */
	TimeDelay(1000);   
	/* Display " B1 : Conversion" message on LCD: Line 1 */ 
	LCD_SendStringByStep(" B1 : Conversion", 1, 1); 
	TimeDelay(100);   
	/* Display "  B2 : LedShow  " message on LCD: Line 2 */ 
	LCD_SendStringByStep("  B2 : LedShow  ", 2, 1); 
	ADCTick = 2;
	while(1)
	{
	    ADC_ConversionCmd(ADC_Conversion_Start);
		PendSemaphore(&ADC_done,0);
		Conversion_Value = ADC_GetConversionValue(ADC_Channel_5);
	    /* Clear the end of conversion flag */
	    ADC_ClearFlag(ADC_FLAG_ECV);
		sprintf((char *)buffer,"ADC:%04d",Conversion_Value);
	    LCD_SetPosCur(2,9);
		LCD_SendStringByStep(buffer,2, 4);
	}
}

void RS232Task(void *a)
{
	int c;

	if(Com0 == NULL) Com0 = CioOpen("COM0",CIO_READ_ONLY | CIO_WRITE_ONLY);
	while(1)
	{
		c = CioGetC(Com0);
		printf("Hey, I Got %c\n",c);
//		CioPutC(Com0,c);
	}
}

void RS232Task2(void *a)
{
	IOCB *Com2;
	int c;
	char *s = NewBlock();		//allocate memory for genning file name
	sprintf(s,"COM%d",(int)a);	//create file name from passed handle
	Com2 = CioOpen(s,CIO_READ_ONLY | CIO_WRITE_ONLY);	//open file
	DeleteBlock(s);				//return memory to pool
	while(1)
	{
		c = CioGetC(Com2);		//get character from com port
		CioPutC(Com2,c);		//put that character right back again
	}
}

void IdleTask(void *arg)
{
	while(1)
	{
		//--------------------------------------------------
		// we don't have anything else better to do, so
		// monitor ADC to see when it is done
		//--------------------------------------------------
   		if(ADC_GetFlagStatus(ADC_FLAG_ECV) != RESET)
		{
			PostSemaphore(&ADC_done,0);
		}
	}
}

void InitRS232Task1(void)
{
	//---------------------------------------------------------------
	// one thing to note here is that I am going to use the same
	// function for two different tasks.  Each instance will have its
	// very own task control block and stack, but the code will be
	// the same.
	//---------------------------------------------------------------
	CreateTask(&Rs232Tcb1,RS232Task2,RS232Stack1,128,3,"RS232 Task 2",(void *)1);
	Insert(&ActiveTasks,&Rs232Tcb1);
}

void InitRS232Task2(void)
{
	//---------------------------------------------------------------
	// one thing to note here is that I am going to use the same
	// function for two different tasks.  Each instance will have its
	// very own task control block and stack, but the code will be
	// the same.
	//---------------------------------------------------------------
	CreateTask(&Rs232Tcb2,RS232Task2,RS232Stack2,128,1,"RS232 Task 2",(void *)2);
	Insert(&ActiveTasks,&Rs232Tcb2);
}

void InitRS232Task(void)
{
	CreateTask(&Rs232Tcb,RS232Task,RS232Stack,128,1,"RS232 Task",(void *)0);
	Insert(&ActiveTasks,&Rs232Tcb);
}

void InitIdleTask(void)
{
	//-------------------------------------------------------------------------
	// Create a task that does things when there is nothing better to do
	//-------------------------------------------------------------------------
	CreateTask(&IdleTcb,IdleTask,IdleStack,128,1,"Idle Task",(void *)0);
	CreateSemaphore(&ADC_done,0,0,"ADC");
	Insert(&ActiveTasks,&IdleTcb);
}

void InitADCTask(void)
{
	CreateTask(&AdcTcb,ADCTask,AdcStack,128,2,"ADC Task",(void *)0);
	Insert(&ActiveTasks,&AdcTcb);
}

void InitTestTask(void)
{
	//-----------------------------------------------------------------
	// create a task that makes a pretty display on the LEDs
	//-----------------------------------------------------------------
	CreateTask(&TtTcb,TestTask,TtStack,64,2,"TestTask",(void *)1);
	CreateSemaphore(&TS_Led,0,1,"Led");
	Insert(&ActiveTasks,&TtTcb);
}

void InitTasks(void)
{
	//------------------------------------
	// initialize all of the tasks
	//------------------------------------
	InitTask();
	InitTestTask();
	InitADCTask();
	InitIdleTask();
	InitRS232Task();
	InitRS232Task1();
	InitRS232Task2();
	//--------------------------------------
	// remove highest priority task from
	// the priority queue and make it
	// ready to execute
	//--------------------------------------
	Delete(&ActiveTasks,(void **)&pxReadyTCB);
	//--------------------------------------
	// Start the RTOS
	//--------------------------------------
	vPortStartFirstTask();
}
